@@ -12,8 +12,8 @@ Simple autocompletion support for HTML and XML documents.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__cvsid__ = "$Id: htmlcomp.py 60378 2009-04-26 04:55:23Z CJP $"
-__revision__ = "$Revision: 60378 $"
+__cvsid__ = "$Id: htmlcomp.py 63004 2009-12-28 04:00:56Z CJP $"
+__revision__ = "$Revision: 63004 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -105,14 +105,14 @@ class Completer(completer.BaseCompleter):
 
         """
         if command in [None, u'']:
-            return u''
+            return list()
 
         buff = self.GetBuffer()
         cpos = buff.GetCurrentPos()
 
         # Check if we are in a php region or not
         if buff.GetStyleAt(cpos) not in HTML_AREA:
-            return u''
+            return list()
 
         cline = buff.GetCurrentLine()
 
@@ -124,7 +124,7 @@ class Completer(completer.BaseCompleter):
                 taglst = _FindXmlTags(buff.GetText())
             else:
                 taglst = TAGS
-            return WrapTagImages(taglst)
+            return completer.CreateSymbols(taglst, completer.TYPE_ELEMENT)
 
         tmp = tmp.rstrip('>').rstrip()
         if len(tmp) and (tmp[-1] in '"\' \t' or tmp[-1].isalpha()):
@@ -147,7 +147,7 @@ class Completer(completer.BaseCompleter):
 
                             if not parts[-1].endswith('>'):
                                 rtag = u">" + rtag
-                            return [rtag, ]
+                            return [ completer.Symbol(rtag, completer.TYPE_ELEMENT) ]
                     break
 
         return list()
@@ -169,11 +169,3 @@ def _FindXmlTags(text):
     else:
         matches = [u'!--', ]
     return matches
-
-def WrapTagImages(taglst):
-    """Apply image identifiers to completion strings
-    @return: list
-
-    """
-    tagstr = u"?%d" % completer.IMG_ELEMENT
-    return [ tag + tagstr for tag in taglst ]
